use NRT::Helper;
use LWP::UserAgent;

package Test;
require Exporter;

sub new {
  my ($classe, %config) = @_;
  my $this = {};
  $classe = ref($classe) || $classe;
  bless ( $this, $classe );
  $this->{url} = "$config{Authentification}{Protocol}$config{Authentification}{Host}$config{Authentification}{Path}";
  $this->{ua} = LWP::UserAgent->new(timeout => 10);
  $this->{ua}->env_proxy;
  $this->{countSuccess} = 0;
  $this->{countFailure} = 0;
  return $this;
}

sub rq {
  my $this = shift ;
  my $path = (exists $_[0])? $_[0] : "";
  my $method = (exists $_[1])? $_[1] : "";
  my $content = (exists $_[2])? $_[2] : "";
  my $authorization = (exists $_[3])? $_[3] : "";

  my $req = HTTP::Request->new( $method, "$this->{url}$path" );
  $req->header( 'Content-Type' => 'application/json' );

  if( $authorization ){
    $req->header(Authorization => $authorization);
  }

  $req->content( $content );
  my $res = $this->{ua}->request($req) ;

  return ( $res->is_success() && $this->is_json( $res->decoded_content ) )?
      $this->json_parse( $res->decoded_content )
      : $res->status_line();
}

sub assertEqual {
  my ($this, $value1, $value2, $equal ) = @_;
  $equal //= 1;
  my $success = 0;

  my $test = ( $value1 eq $value2 );
  $test = ( $equal )? $test : !$test;
  $assertedComparator = ($equal)? "equal" : "NOT equal" ;
  $unassertedComparator = ($equal)? "NOT equal" : "equal" ;

  if( $test )
  {
    $this->record_result( 1 );
    print " $value1 ";
    Helper::printColor( "bright_blue", $assertedComparator);
    print " $value2\n";
    return 1;
  }else{
    $this->record_result( 0 );
    print " $value1 ";
    Helper::printColor( "bright_blue", $assertedComparator);
    print " $value2\n";
    return 0;
  };
}

sub assertNotEqual {
  my $this = shift;
  return $this->assertEqual( $_[0], $_[1], 0 );
}

sub get_code {
  my ($this, $path) = @_;
  if(!defined $path){
    $path = "";
    #exit 1;
  }
  my $response = $this->{ua}->get( "$this->{url}$path" );

  if ($response->is_success) {
    return $response->decoded_content;
  }else{
    return $response->status_line;
  }
}

sub record_result {
  my $this = shift ;
  my $status = (exists $_[0])? $_[0] : false;

  if( $status ){
    Helper::printColor( "green", "Success :");
    $this->{countSuccess}++;
  }else{
    Helper::printColor( "red", "Failure :");
    $this->{countFailure}++;
  }
}

sub json_parse {
  my ($this, $json_text) = @_;
  my $json = JSON->new;
  $res = eval{ $json->decode("$json_text") };
  return ($@)?
    0 :
    $res ;
}

sub is_json {
  my ($this, $json_text) = @_;
  my $json = JSON->new;
  $res = eval{ $json->decode("$json_text") };
  return ($@)? 0 : 1 ;
}

sub get_countSuccess {
  my $this = shift;
  return $this->{countSuccess} ;
}

sub get_countFailure{
  return $countFailure ;
}

sub get_url{
  my $this = shift;
  return $this->{url} ;
}

sub test_rq_empty {
  my $this = shift;
  my $rq = $this->get_code();
  return $this->assertEqual( $rq, "404 Not Found" );
}

sub test_user_authentification {
  my ($this, $user) = @_;
  my $auth = $user->authentification;
  return $this->assertEqual( $auth->{email}, $user->{email} );
}

sub test_user_exists {
  my ($this, $user) = @_;
  my $ruser = $user->fetch;
  return $this->assertEqual( $ruser->{data}{attributes}{email}, $user->{email} );
}

sub test_user_not_exists {
  my ($this, $user) = @_;
  return $this->assertNotEqual( $ruser->{data}{attributes}{email}, $user->{email} );
}

sub test_user_create {
  my ($this, $user) = @_;
  my $creation = $user->create();
  return $this->assertEqual( $creation->{data}{attributes}{email}, $user->{email} );
}

sub test_user_already_removed {
  my ($this, $user) = @_;
  return $this->assertEqual($user->remove, '404 Not Found');
}

sub test_user_removed {
  my ($this, $user) = @_;
  my $user = $user->fetch();
  return $this->assertEqual( $user, '404 Not Found' );
}

sub test_user_remove {
  my ($this, $user) = @_;
  return $this->assertEqual( $user->remove, '204 No Content');
}

sub test_user_already_created {
  my ($this, $user) = @_;
  my $creation = $user->fetch;
  return $this->assertEqual( $creation->{data}{id}, $user->get_id);
}

sub test_create_activity {
  my ($this, $activity) = @_;
  my $res = $activity->create;
  return $this->assertEqual( $res->{data}{attributes}{name}, $activity->{name} );
}

sub test_activity_remove {
  my ($this, $activity) = @_;
  $activity->remove;
  return $this->assertNotEqual( $ractivity->{data}{id}, $activity->{id} );
}

sub test_activity_exists {
  my ($this, $activity) = @_;
  my $ractivity = $activity->fetch;
  return $this->assertEqual( $ractivity->{data}{id}, $activity->{id} );
}

sub test_activity_not_exists {
  my ($this, $activity) = @_;
  my $ractivity = $activity->fetch;
  return $this->assertNotEqual( $ractivity->{data}{id}, $activity->{id} );
}

sub test_create_task {
  my ($this, $task) = @_;
  my $res = $task->create;
  return $this->assertEqual( $res->{data}{attributes}{name}, $task->{name} );
}

sub test_task_exists {
  my ($this, $task) = @_;
  my $rtask = $task->fetch;
  return $this->assertEqual(  $rtask->{data}{id}, $task->{id} );
}

sub test_task_not_exists {
  my ($this, $task) = @_;
  my $rtask = $task->fetch;
  eval {exists $rtask->{data}{id}};
  return $this->assertNotEqual( $@, 0 );
}

sub test_task_remove {
  my ($this, $task) = @_;
  my $rtask = $task->remove;
  return $this->assertNotEqual( $rtask->{data}{id}, $task->{id} );
}

return 1;
